#!/usr/local/bin/perl -w

# perl SeqConcatBD.pl

# fasta 形式のファイルを concatenete します．
# SpList に種名 (OTU 名) をリストアップしてください．
# Infile_fol にファスタ形式のファイルを入れておきます．

# 1 種づつ配列を取り出してアウトファイルに保存します．
# 大量データ用でメモリを使わないように作成しました．


use strict;
use warnings;

#####
my $SpListFile = "SpList.txt";                # Concatenate した配列を作りたい種名をリストアップする．
my $InfileFol = "Infile_fol";                 # 複数のシーケンスファイルを入れておくフォルダの名前
my $OutPutFile1 = "Out_Concatenated.fas";     # アウトプットファイルの名前
#####

my $gap = "";
my $name = "";
my $seq = "";
my $addname = "";
my $SeqLength = 0;
my $LineMilestone = 0;

my @SpListArray = ( );   # SpList.txt から得た全 OTU 名のリスト
my @InFiles = ( );       # Infile_Fol に入っている遺伝子配列ファイルのリスト
my @inFileCont = ( );    # 遺伝子ごとにファイルを格納する配列

my %efas = ( );          # 遺伝子ファイルごとに得られたファスタ形式を格納するハッシュ
my @efasNames = ( );     # 遺伝子ファイルごとに得られた name line を格納するハッシュ
#######


#### インファイルの読み込み．
# $SpListFile にある種名を読んで配列に入れる．
open(SPLIST,"$SpListFile");
@SpListArray = <SPLIST>;
close SPLIST;

# infile_fol にあるインファイルの名前をアレイに入れる．
opendir(DIR,"$InfileFol");
my @InFilesRaw = readdir(DIR);
close(DIR);

foreach (@InFilesRaw) {
  if ($_ !~ /^\./) {
    push(@InFiles,$_);
  }
}

### スクリーンアウト
print "\n### SpList: ".scalar@SpListArray." spp\n\n";
print "\n### geneList: ".scalar@InFiles." genes\n";
foreach (@InFiles) {
  print "$_\n";
}
print "\n";


######### Main スクリプト ########
open(OUT,">$OutPutFile1");                      # アウトファイルを開く．
foreach my $spname (@SpListArray) {             # 種名リストを 1 種づつ処理する．
  chomp($spname);
#  my $spnamePS = ">".$spname;
  print OUT ">$spname\n";                       # 種名をプリントアウト
  print "$spname\n";

  # infile_fol に入っている Fasta file を一つずつ処理する．
  foreach my $inFileName (@InFiles) {           
    if ($inFileName !~ /^\./) {                  # . で始まるファイル以外をあける．

      open(INFILE,"$InfileFol/$inFileName");     # infile の情報を取り込む
      @inFileCont = <INFILE>;
      close INFILE;

     # インファイルの内容が入った配列をサブルーチンに渡す．データがハッシュ，NameLine が配列として帰ってくる．最後の配列の長さも返ってくる．
     my ($ref_fasHashSR,$ref_nameArraySR,$seqLength) = &fasHashReader(\@inFileCont);
       %efas = %$ref_fasHashSR;
       @efasNames = @$ref_nameArraySR;
       
    
     ### プリントアウト
     if(grep/^>$spname$/,@efasNames) {                             ######### SpList の種名が各ファイルにあるか検証している．「>」の name line が SpList とかわっていたら，/^>$spnamePS / を変更する．
       my @temp = grep(/^>$spname$/,keys %efas);                   ##### 上の行と同様．
       my $temp = shift @temp;
       print "$inFileName: FOUND, seqLength: ".length($efas{$temp})."\n";
#          print OUT "     $inFileName     ";                                       # この行をアクティブにすると，遺伝子名が配列の先頭に書かれます．確認用です．
       print OUT "$efas{$temp}";
     } else {                                                                   # 遺伝子ファイルに種名がない場合は，ギャップを入れる．
       my $gapSeq = "-" x $seqLength;
       print "$inFileName: NOT, gapLength: ".length($gapSeq)."\n";
#          print OUT "     $inFileName     ";                                       # この行をアクティブにすると，遺伝子名が配列の先頭に書かれます．確認用です．
       print OUT "$gapSeq";
     }
    }
  }
  print OUT "\n";
  print "\n";
}

exit;


####### サブルーチン 


#  Fasta 形式をハッシュに読み込む Subroutine
sub fasHashReader
{
my $name = "";
my $seq = "";
my $seqLength = "";
my $stone = 0;
my @nameArraySR = ( );
my %fasHashSR = ( );

my ($inFileContSR) = @_;

foreach my $inFileLine (@$inFileContSR) {
  chomp($inFileLine);
  if($inFileLine =~ /^>/){  
    if($stone == 0){
      $name = $inFileLine;
      $stone = 1;  
    } else {
      $fasHashSR{$name} = $seq;  
      push(@nameArraySR,$name);
      $name = $inFileLine;
      $seq = "";
    }
  } else {
    $seq .= $inFileLine; 
  }
}
$fasHashSR{$name} = $seq;
$seqLength = length$seq;
push(@nameArraySR,$name);

# ハッシュと配列をメインプログラムに返す．
return (\%fasHashSR,\@nameArraySR,$seqLength);

}

